/*= OX802 Metrix ===================================================== */
/* LabWindows Instrument Driver for OX802.                             */
/* Original Release: November/29/95                                    */
/* By:  H. CHOL & F. VALOTEAU                                          */
/* Originally written in C                                             */
/* Modification History:                                               */
/*=====================================================================*/
#include "formatio.h"
#include "rs232.h"
#include "ox802.h"


/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module.  */
/* vert_mode contains the current vertical mode of the instrument.         */
/* scope contains the hardware version of the front panel microprocessor   */
/* ox802_err: the error variable for the instrument module                 */
/*=========================================================================*/
static short port;
static short vert_mode;
static short ox802_err;


/*= UTILITY ROUTINES ======================================================*/
int ox802_invalid_short_range (short val, short min, short max, int err_code);
int ox802_invalid_integer_range (int val, int min, int max, int err_code);
int ox802_invalid_longint_range (long val, long min, long max, int err_code);
int ox802_invalid_real_range (double val, double min, double max, int err_code);
int ox802_device_closed (void);
int ox802_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox802_init (int rs_port,int param)
{
    char identifier[3];

    ox802_err = 0;
    if (ox802_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (param,0,1,-2) != 0){
        return ox802_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 0, 0);
    if (rs232err != 0) {
        ox802_err = rs232err+300;
        return ox802_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox802_err = rs232err+300;
        return ox802_err;
    }
    /*   Sets XON/XOFF ON   */
    SetXMode (rs_port, 1);
    if (rs232err != 0) {
        ox802_err = rs232err+300;
        return ox802_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox802_send_command (OX802_ASK_ECHO, 0) !=0 ){
        ox802_close();
        return ox802_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (identifier[1]!=0x34){
        ox802_close();
        return 223;
    }
    else{
        /* Lock the front panel */
        if (ox802_send_command (OX802_REMOTE, param) != 0) {
        return ox802_err;
        }
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    CH1           : AC    1V/Div      */
    /*    Com NOR/DIFF  :   NOR             */
    /*    CH2           : AC    1V/Div      */
    /*    Time base     : x10 OFF,0.1MS/Div,DELAY OFF                       */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC     */
    vert_mode = 1; /* ALT */
    if (ox802_display (1,0) != 0){
        return ox802_err;
    }
    if (ox802_channel (0, 0, 7,1) != 0){
        return ox802_err;
    }
    if (ox802_channel (1, 0, 7,1) !=0){
        return ox802_err;
    }
    if (ox802_timebase (0, 10, 0) != 0){
        return ox802_err;
    }
    if (ox802_trigger (1, 0, 1, 0, 0) != 0){
        return ox802_err;
    }
    return ox802_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox802_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[15];
    char    buff[16];

    tab_fonct[0] = OX802_VERTICAL_MODE;
    tab_fonct[1] = OX802_RANGE_CH2;
    tab_fonct[2] = OX802_COUPLING_CH1;
    tab_fonct[3] = OX802_COUPLING_CH2;
    tab_fonct[4] = OX802_RANGE_CH1;
    tab_fonct[5] = OX802_COM_NOR_DIFF;
    tab_fonct[6] = OX802_RANGE_TIME_BASE;
    tab_fonct[7] = OX802_EXP10;
    tab_fonct[8] = OX802_TRIGGER_SOURCE;
    tab_fonct[9] = OX802_TRIGGER_AUTO;
    tab_fonct[10] = OX802_TRIGGER_COUPLING;
    tab_fonct[11] = OX802_TRIGGER_PTP;
    tab_fonct[12] = OX802_TRIGGER_SLOPE;
    tab_fonct[13] = OX802_TB_DELAY;
    tab_fonct[14] = OX802_BEAM_FINDER;

    ox802_err = 0;
    if (ox802_device_closed() != 0 ){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox802_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox802_err =  -2;
            return ox802_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox802_err = rs232err+300;
            return ox802_err;
        }
        if (ox802_send_command (OX802_READ_SETUP, 0) != 0 ){
            return ox802_err;
        }
        i = 0;
        while (i < 15)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox802_err = rs232err+300;
                return ox802_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox802_err = rs232err+300;
                return ox802_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox802_err = rs232err+300;
                return ox802_err;
            }
            i++;
        }
        buff[15] = buff[4];
        buff[4] = buff[0];
        buff[0] = buff[15];
        buff[15] = 0;
        if (WriteFile (file_id, buff, 15) == -1){
            ox802_err =  230;
            return ox802_err;
        }
        if (CloseFile (file_id) == -1){
            ox802_err = -2 ;
            return ox802_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox802_err =  -2;
            return ox802_err;
        }
        if (ReadFile (file_id, buff, 15) ==  -1)  {
            ox802_err =  231;
            return ox802_err;
        }
        i = 0;
        vert_mode = buff[0] - 0x30;
        while (i < 15)  {
            if (ox802_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox802_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox802_err =  -2;
            return ox802_err;
        }

    }
    return ox802_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox802_display (int mod,int beam)
{
    ox802_err = 0;
    if (ox802_device_closed() != 0 ){
        return ox802_err ;
    }
    if (ox802_invalid_integer_range (mod,0,5,-1) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (beam,0,1,-2) != 0){
        return ox802_err;
    }
    if (ox802_send_command (OX802_VERTICAL_MODE, mod) != 0){
        return ox802_err;
    }
    vert_mode = mod;
    /* Beam Finder */
    if (ox802_send_command (OX802_BEAM_FINDER, beam) != 0){
        return ox802_err;
    }
    return ox802_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox802_channel (int channel_num, int coupling, int sensibility,int nor_diff)
{
    ox802_err = 0;
    if (ox802_device_closed() != 0 ){
        return ox802_err ;
    }
    if (ox802_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (sensibility,0,13,-3) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (nor_diff,0,1,-4) != 0){
        return ox802_err;
    }
    if (!channel_num){
        if (ox802_send_command (OX802_COUPLING_CH1, coupling) != 0)
            return ox802_err;
    }
    else{
        if (ox802_send_command (OX802_COUPLING_CH2, coupling) != 0)
            return ox802_err;
    }
    if (!channel_num){
        if (ox802_send_command (OX802_RANGE_CH1, sensibility) != 0)
            return ox802_err;
    }
    else{
        if (ox802_send_command (OX802_RANGE_CH2, sensibility) != 0){
            return ox802_err;
        }
    }
    /* Input NOR/DIFF */
    if (ox802_send_command (OX802_COM_NOR_DIFF, nor_diff) !=0){
        return ox802_err;
    }

    return ox802_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox802_timebase (int expend,int range,int dlay)
{

    ox802_err = 0;
    if (ox802_device_closed() != 0 ){
        return ox802_err ;
    }
    if (ox802_invalid_integer_range (expend,0,1,-1) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (range,0,17,-2) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox802_err;
    }
    /* if XY mode, error -4 */
    if (vert_mode == 5)  {
        ox802_err =  -4;
        return ox802_err ;
    }
    if (ox802_send_command (OX802_EXP10, expend) != 0){
        return ox802_err;
    }
    if (ox802_send_command (OX802_RANGE_TIME_BASE, range) != 0){
        return ox802_err;
    }
    if (ox802_send_command (OX802_TB_DELAY, dlay) != 0){
        return ox802_err;
    }
    return ox802_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox802_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox802_err = 0;
    if (ox802_device_closed() != 0 ){
        return ox802_err ;
    }
    if (ox802_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (slope,0,1,-2) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (source,0,4,-4) != 0){
        return ox802_err;
    }
    if (ox802_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox802_err;
    }
    /* if XY mode, error= -6 */
    if (vert_mode == 5)  {
        ox802_err =  -6;
        return ox802_err ;
    }
    if (ox802_send_command (OX802_TRIGGER_SOURCE, source) != 0){
        return ox802_err;
    }
    if (source != 3){
        if (ox802_send_command (OX802_TRIGGER_COUPLING, coupling) != 0){
            return ox802_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox802_send_command (OX802_TRIGGER_COUPLING, 6) != 0){
            return ox802_err;
        }
    }
    if (ox802_send_command (OX802_TRIGGER_PTP, ptp) != 0){
        return ox802_err;
    }
    if (ox802_send_command (OX802_TRIGGER_AUTO, mod_auto) != 0){
        return ox802_err;
    }
    if (ox802_send_command (OX802_TRIGGER_SLOPE, slope) != 0){
        return ox802_err;
    }
    return ox802_err;
}
/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox802_close (void)
{
    /*  Check for device closed */
    if (ox802_device_closed() !=0){
        return ox802_err;
    }

    /* Unlock front panel */
    if (ox802_send_command (OX802_REMOTE, 0) != 0){
        return ox802_err;
    }
    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox802_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox802_err = rs232err+300;
        return ox802_err;
    }
    port = 0;
    return ox802_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox802_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox802_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox802_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox802_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox802_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox802_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox802_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox802_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox802_device_closed (void)
{
    if (port== 0){
        ox802_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the scope. If the scope is not*/
/*          initialized, an error 232 is returned.                         */
/*=========================================================================*/
int ox802_send_command (int function, int parameter)
{
    ox802_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox802_err = 230;
            return ox802_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox802_err = 230;
            return ox802_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox802_err = 230;
            return ox802_err ;
        }
    }
    else{
        ox802_err = 232;
    }
    return ox802_err ;
}
/*= THE END ===============================================================*/
